##############################################################################
# dictool.tcl - multilingual dictionary tool
#
# Copyright (C) 2002 Reiji SUZUKI
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##############################################################################
#

#!/bin/sh
# the next line restarts using wish\
exec wish "$0" "$@" 

if {![info exists vTcl(sourcing)]} {

    switch $tcl_platform(platform) {
	windows {
            option add *Button.padY 0
	}
	default {
	    option add *Scrollbar.width 10
            option add *Scrollbar.highlightThickness 0
            option add *Scrollbar.elementBorderWidth 2
            option add *Scrollbar.borderWidth 2
	}
    }
    
}

#############################################################################
# Visual Tcl v1.60 Project
#

#############################################################################
# vTcl Code to Load Stock Fonts


if {![info exist vTcl(sourcing)]} {
set vTcl(fonts,counter) 0
#############################################################################
## Procedure:  vTcl:font:add_font

proc {vTcl:font:add_font} {font_descr font_type {newkey {}}} {
     global vTcl

     incr vTcl(fonts,counter)
     set newfont [eval font create $font_descr]

     lappend vTcl(fonts,objects) $newfont

     # each font has its unique key so that when a project is
     # reloaded, the key is used to find the font description

     if {$newkey == ""} {
          set newkey vTcl:font$vTcl(fonts,counter)

          # let's find an unused font key
          while {[vTcl:font:get_font $newkey] != ""} {
             incr vTcl(fonts,counter)
             set newkey vTcl:font$vTcl(fonts,counter)
          }
     }

     set vTcl(fonts,$newfont,type)                      $font_type
     set vTcl(fonts,$newfont,key)                       $newkey
     set vTcl(fonts,$vTcl(fonts,$newfont,key),object)   $newfont

     lappend vTcl(fonts,$font_type) $newfont

     # in case caller needs it
     return $newfont
}

#############################################################################
## Procedure:  vTcl:font:get_font

proc {vTcl:font:get_font} {key} {
    global vTcl
    if {[info exists vTcl(fonts,$key,object)]} then {
        return $vTcl(fonts,$key,object)
    } else {
        return ""
    }
}

}
#############################################################################
# vTcl Code to Load User Fonts
#vTcl:font:add_font \
#    "-family $lary(0,0) -size $lary(0,1) -weight normal -slant roman -underline 0 -overstrike 0" \
#    user \
 #   vTcl:font10
#################################
# VTCL LIBRARY PROCEDURES
#

if {![info exists vTcl(sourcing)]} {
#############################################################################
## Library Procedure:  Window

proc {Window} {args} {
    global vTcl
    set cmd     [lindex $args 0]
    set name    [lindex $args 1]
    set newname [lindex $args 2]
    set rest    [lrange $args 3 end]
    if {$name == "" || $cmd == ""} { return }
    if {$newname == ""} { set newname $name }
    if {$name == "."} { wm withdraw $name; return }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists} {
                wm deiconify $newname
            } elseif {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[winfo exists $newname] && [wm state $newname] == "normal"} {
                vTcl:FireEvent $newname <<Show>>
            }
        }
        hide    {
            if {$exists} {
                wm withdraw $newname
                vTcl:FireEvent $newname <<Hide>>
                return}
        }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}
#############################################################################
## Library Procedure:  vTcl:DefineAlias

proc {vTcl:DefineAlias} {target alias widgetProc top_or_alias cmdalias} {
    global widget
    set widget($alias) $target
    set widget(rev,$target) $alias
    if {$cmdalias} {
        interp alias {} $alias {} $widgetProc $target
    }
    if {$top_or_alias != ""} {
        set widget($top_or_alias,$alias) $target
        if {$cmdalias} {
            interp alias {} $top_or_alias.$alias {} $widgetProc $target
        }
    }
}
#############################################################################
## Library Procedure:  vTcl:DoCmdOption

proc {vTcl:DoCmdOption} {target cmd} {

    ## menus are considered toplevel windows
    set parent $target
    while {[winfo class $parent] == "Menu"} {
        set parent [winfo parent $parent]
    }

    regsub -all {\%widget} $cmd $target cmd
    regsub -all {\%top} $cmd [winfo toplevel $parent] cmd

    uplevel #0 [list eval $cmd]
}
#############################################################################
## Library Procedure:  vTcl:FireEvent

proc {vTcl:FireEvent} {target event {params {}}} {
    ## The window may have disappeared
    if {![winfo exists $target]} return
    ## Process each binding tag, looking for the event
    foreach bindtag [bindtags $target] {
        set tag_events [bind $bindtag]
        set stop_processing 0
        foreach tag_event $tag_events {
            if {$tag_event == $event} {
                set bind_code [bind $bindtag $tag_event]
                foreach rep "\{%W $target\} $params" {
                    regsub -all [lindex $rep 0] $bind_code [lindex $rep 1] bind_code
                }
                set result [catch {uplevel #0 $bind_code} errortext]
                if {$result == 3} {
                    ## break exception, stop processing
                    set stop_processing 1
                } elseif {$result != 0} {
                    bgerror $errortext
                }
                break
            }
        }
        if {$stop_processing} {break}
    }
}
#############################################################################
## Library Procedure:  vTcl:Toplevel:WidgetProc

proc {vTcl:Toplevel:WidgetProc} {w args} {
    if {[llength $args] == 0} {
        ## If no arguments, returns the path the alias points to
        return $w
    }
    ## The first argument is a switch, they must be doing a configure.
    if {[string index $args 0] == "-"} {
        set command configure
        ## There's only one argument, must be a cget.
        if {[llength $args] == 1} {
            set command cget
        }
    } else {
        set command [lindex $args 0]
        set args [lrange $args 1 end]
    }
    switch -- $command {
        "hide" - "Hide" - "show" - "Show" {
            Window [string tolower $command] $w
        }
        "ShowModal" {
            Window show $w
            raise $w
            grab $w
            tkwait window $w
            grab release $w
        }
        default {
            uplevel $w $command $args
        }
    }
}
#############################################################################
## Library Procedure:  vTcl:WidgetProc

proc {vTcl:WidgetProc} {w args} {
    if {[llength $args] == 0} {
        ## If no arguments, returns the path the alias points to
        return $w
    }
    ## The first argument is a switch, they must be doing a configure.
    if {[string index $args 0] == "-"} {
        set command configure
        ## There's only one argument, must be a cget.
        if {[llength $args] == 1} {
            set command cget
        }
    } else {
        set command [lindex $args 0]
        set args [lrange $args 1 end]
    }
    uplevel $w $command $args
}
#############################################################################
## Library Procedure:  vTcl:toplevel

proc {vTcl:toplevel} {args} {
    uplevel #0 eval toplevel $args
    set target [lindex $args 0]
    namespace eval ::$target {}
}
}


if {[info exists vTcl(sourcing)]} {

proc vTcl:project:info {} {
    set base .ktop
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 1
    }
    namespace eval ::widgets::$base.kbd {
        array set save {-background 1 -highlightbackground 1 -highlightcolor 1}
    }

    set base .top39
    namespace eval ::widgets::$base {
        set set,origin 1
        set set,size 1
    }
    namespace eval ::widgets::$base.tex40 {
        array set save {-background 1 -font "\"$lary(0,0)\" $lary(0,1)" -insertbackground 1 -yscrollcommand 1}
    }
    namespace eval ::widgets::$base.lab42 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.ent45 {
        array set save {-background 1 -font "\"$lary(0,0)\" $lary(0,1)" -insertbackground 1 -textvariable 1}
    }
    namespace eval ::widgets::$base.lab46 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.but70 {
        array set save {-command 1 -pady 1 -text 1}
    }
    namespace eval ::widgets::$base.lab71 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.lab72 {
        array set save {-text 1}
    }
    namespace eval ::widgets::$base.che39 {
        array set save {-selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.scr40 {
        array set save {-command 1}
    }
    namespace eval ::widgets::$base.che40 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che41 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che43 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che44 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che45 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che46 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che47 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che48 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -textvariable 1 -variable 1}
    }
    namespace eval ::widgets::$base.che49 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che50 {
        array set save {-_tooltip 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che51 {
        array set save {-command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che52 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che53 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che54 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che56 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.che57 {
        array set save {-_tooltip 1 -command 1 -selectcolor 1 -text 1 -variable 1}
    }
    namespace eval ::widgets::$base.but61 {
        array set save {-command 1 -disabledforeground 1 -pady 1 -text 1}
    }
    namespace eval ::widgets::$base.but62 {
        array set save {-command 1 -disabledforeground 1 -pady 1 -text 1}
    }
    namespace eval ::widgets::$base.but63 {
        array set save {-command 1 -disabledforeground 1 -height 1 -pady 1 -text 1 -width 1}
    }
    namespace eval ::widgets::$base.but64 {
        array set save {-command 1 -disabledforeground 1 -height 1 -pady 1 -text 1 -width 1}
    }
    namespace eval ::widgets::$base.m62 {
        array set save {-activeborderwidth 1 -borderwidth 1 -tearoff 1}
    }
    namespace eval ::widgets_bindings {
        set tagslist {_TopLevel _vTclBalloon}
    }
    namespace eval ::vTcl::modules::main {
        set procs {
            init
            main
            clist2list
            keyboard
            search
            vTclWindow.
            vTclWindow.ktop
            vTclWindow.top39
        }
    }
}
}

#################################
# USER DEFINED PROCEDURES
#
#############################################################################
## Procedure:  main

proc {main} {argc argv} {

}
#############################################################################
## Procedure:  clist2list

proc {clist2list} {clist} {
    #-- clist: compact integer list w.ranges, e.g. {1-5 7 9-11}
    set res {}
    foreach i $clist {
	if [regexp {([^-]+)-([^-]+)} $i -> from to] {
	    for {set j [expr $from]} {$j<=[expr $to]} {incr j} {
		lappend res $j
	    }
	} else {lappend res [expr $i]}
    }
    set res
 }
#############################################################################
## Procedure:  keyboard

proc {keyboard} {w args} {
    destroy $w
    frame $w
global lary
    array set opts {-keys {0x0061-0x0066 0x0278 0x0067 0x0263 0x0069-0x006A 0x02B2 0x006B-0x006C 0x026C 0x006D-0x006E 0x014B 0x006F-0x0073 0x0283 0x0074-0x0078 0x03C7 0x007A 0x0292 0x0259 0x0295 0x0294 0x0027} -keysperline 18}
    array set opts $args ;# no errors checked
    set klist {}; set n 0
    foreach i [clist2list $opts(-keys)] {
	set c [format %c $i]
	set cmd "$opts(-receiver) insert insert [list $c]"
	if {$i>=0x5D0 && $i<=0x6FF} {
	    append cmd ";$opts(-receiver) mark set insert {insert - 1 chars}"
	}
	button $w.k$i -font "\"$lary(0,0)\" $lary(0,1)" -text $c -command $cmd  -padx 5 -pady 0
	lappend klist $w.k$i
	if {[incr n]==$opts(-keysperline)} {
	    eval grid $klist -sticky news
	    set n 0; set klist {}
	}
    }
    if [llength $klist] {eval grid $klist -sticky news}
    pack $w -side bottom
    set w ;# return widget pathname, as the others do
}
#############################################################################
## Procedure:  search

proc {search} {} {
global widget
global keyword
global isWord
global lf
global lh
global lt
global ary
global maxl

Text1 configure -state normal
Text1 delete 1.0 end
#array unset sary
if {$keyword == ""} {
 return
}

set tk1 "$keyword"

regsub -all {(\W)} $tk1 {\\\1} tk1

if {$isWord == 1} {
    set tk1 "\\W$tk1\\W"
}


set c 1
set everMatched 0
for {set i 0} {$i < ( $maxl - 1 )} { incr i} {
        set isMatched 0
        for {set k 1} {$k < 9} { incr k} {
            if {$lf($k) == 1} {
            
            if {[ regexp $tk1 $ary($i,$k) Match ] } {
                set isMatched 1
                set everMatched 1
                set mt $k
                set k 9
            }
            }
        }
        
        if { $isMatched == 1} {
        
        set tmp ""
        append tmp "\[" $lh($mt) "\]" $ary($i,$mt) "\n"
        for {set k 1} {$k < 9} { incr k} {
            if {$lt($k) == 1} {
                append tmp "\t\[" $lh($k) "\]" $ary($i,$k) "\n"
            }
        }
        append tmp "\n"
        Text1 insert end $tmp
        incr c
        }
}

if {$everMatched == 0} {
    Text1 inser end "No hits."
}

Text1 configure -state disabled
}


#############################################################################
## Procedure:  changekey

proc {changekey} {no} {
global widget
global lary
keyboard .ktop.kbd -keys $lary([expr $no + 11],1) -receiver Entry1
}

#############################################################################
## Initialization Procedure:  init

proc {init} {argc argv} {
global widget

global from
global to
global ary
global dc
global lines
global lf
global lh
global keyword
global keyname
global tt 
global lary
global maxl

        set fn "./dictool_config.dat"
	set f [open $fn]
	fconfigure $f -encoding "utf-8"
	regsub -all \uFEFF [read $f [file size $fn]] "" text
	set i 0
	foreach line [split $text \n] {
	    set lines($i) "$line"
	    set j 0
	    foreach res [split $line ","] {
	    set lary($i,$j) "$res"
	        incr j
	    }
            incr i
	}
	close $f
#set from 1
set to 4
set dc 0
set tt 1

        set fn "./goishuu.txt"
	set f [open $fn]
	fconfigure $f -encoding "utf-8"
	regsub -all \uFEFF [read $f [file size $fn]] "" text
	set i 0
	foreach line [split $text \n] {
	    set lines($i) " $line "
	    set j 0
	    foreach res [split $line \t] {
	    set ary($i,$j) " $res "
	        incr j
	    }
            incr i
	}
	close $f
	set maxl $i

# write keyboard scripts here!

toplevel .ktop -menu .ktop.menu
menu .ktop.menu

wm title .ktop $lary(9,0)

menu .ktop.menu.lang
keyboard .ktop.kbd -receiver Entry1

.ktop.menu add cascade -label $lary(1,0) -menu .ktop.menu.lang
.ktop.menu.lang add command -label $lary(12,0) -command  [list keyboard .ktop.kbd -keys $lary(12,1) -receiver Entry1 ]
.ktop.menu.lang add command -label $lary(13,0) -command  [list keyboard .ktop.kbd -keys $lary(13,1) -receiver Entry1 ]
.ktop.menu.lang add command -label $lary(14,0) -command  [list keyboard .ktop.kbd -keys $lary(14,1) -receiver Entry1 ]
.ktop.menu.lang add command -label $lary(15,0) -command  [list keyboard .ktop.kbd -keys $lary(15,1) -receiver Entry1 ]
pack .ktop.kbd

set lh(1) $lary(10,0)
set lh(2) $lary(10,1)
set lh(3) $lary(10,2)
set lh(4) $lary(10,3)
set lh(5) $lary(10,4)
set lh(6) $lary(10,5)
set lh(7) $lary(10,6)
set lh(8) $lary(10,7)

}

init $argc $argv

#################################
# VTCL GENERATED GUI PROCEDURES
#
proc vTclWindow.top39 {base} {
global lary
    if {$base == ""} {
        set base .top39
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    set top $base
    ###################
    # CREATING WIDGETS
    ###################
    vTcl:toplevel $base -class Toplevel \
        -menu "$base.m62" -highlightbackground #d9d9d9 -highlightcolor black 
    wm focusmodel $base passive
    wm geometry $base 680x470+37+206; update
    wm maxsize $base 1028 753
    wm minsize $base 104 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base $lary(8,0)
    vTcl:DefineAlias "$base" "Toplevel1" vTcl:Toplevel:WidgetProc "" 1
    bindtags $base "$base Toplevel all _TopLevel"
    vTcl:FireEvent $base <<Create>>
    wm protocol $base WM_DELETE_WINDOW "vTcl:FireEvent $base <<DeleteWindow>>"

    text $base.tex40 \
        -background white -font "\"$lary(0,0)\" $lary(0,1)" \
        -insertbackground black -yscrollcommand {Scrollbar1 set} \
	-state disabled
    vTcl:DefineAlias "$base.tex40" "Text1" vTcl:WidgetProc "Toplevel1" 1

    label $base.lab42 \
        -text $lary(1,0): 
    vTcl:DefineAlias "$base.lab42" "Label1" vTcl:WidgetProc "Toplevel1" 1
    entry $base.ent45 \
        -background white -font "\"$lary(0,0)\" $lary(0,1)" \
        -insertbackground black -textvariable keyword 
    vTcl:DefineAlias "$base.ent45" "Entry1" vTcl:WidgetProc "Toplevel1" 1
    bind $base.ent45 <Key-Return> {
        # TODO: your event handler here
search
    }
    label $base.lab46 \
        -text $lary(5,0)
    vTcl:DefineAlias "$base.lab46" "Label3" vTcl:WidgetProc "Toplevel1" 1
    button $base.but70 \
        -command search -pady 0 -text $lary(6,0)
    vTcl:DefineAlias "$base.but70" "Button1" vTcl:WidgetProc "Toplevel1" 1
    label $base.lab71 \
        -text "($lary(4,0))"
    vTcl:DefineAlias "$base.lab71" "Label4" vTcl:WidgetProc "Toplevel1" 1
    label $base.lab72 \
        -text "($lary(4,1))"
    vTcl:DefineAlias "$base.lab72" "Label5" vTcl:WidgetProc "Toplevel1" 1
    checkbutton $base.che39 \
        -selectcolor #ffffff -text $lary(7,0) -variable isWord 
    vTcl:DefineAlias "$base.che39" "Checkbutton1" vTcl:WidgetProc "Toplevel1" 1
    scrollbar $base.scr40 \
        -command {Text1 yview} 
    vTcl:DefineAlias "$base.scr40" "Scrollbar1" vTcl:WidgetProc "Toplevel1" 1
    checkbutton $base.che40 \
        -command {#set lf(1) [ expr 1 - $lf(1) ]} -selectcolor #ffffff \
        -text $lary(10,0) -textvariable {} -variable lt(1) 
    vTcl:DefineAlias "$base.che40" "Checkbutton2" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che40 "$base.che40 Checkbutton $top all _vTclBalloon"
    bind $base.che40 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,0)
    }
    checkbutton $base.che41 \
        -selectcolor #ffffff -text $lary(10,1) -textvariable {} -variable lt(2) 
    vTcl:DefineAlias "$base.che41" "Checkbutton3" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che41 "$base.che41 Checkbutton $top all _vTclBalloon"
    bind $base.che41 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,1)
    }
    checkbutton $base.che43 \
        -selectcolor #ffffff -text $lary(10,2) -textvariable {} -variable lt(3) 
    vTcl:DefineAlias "$base.che43" "Checkbutton5" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che43 "$base.che43 Checkbutton $top all _vTclBalloon"
    bind $base.che43 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,2)
    }
    checkbutton $base.che44 \
        -selectcolor #ffffff -text $lary(10,3) -textvariable {} -variable lt(4) 
    vTcl:DefineAlias "$base.che44" "Checkbutton6" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che44 "$base.che44 Checkbutton $top all _vTclBalloon"
    bind $base.che44 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,3)
    }
    checkbutton $base.che45 \
        -selectcolor #ffffff -text $lary(10,4) -textvariable {} -variable lt(5) 
    vTcl:DefineAlias "$base.che45" "Checkbutton7" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che45 "$base.che45 Checkbutton $top all _vTclBalloon"
    bind $base.che45 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,4)
    }
    checkbutton $base.che46 \
        -selectcolor #ffffff -text $lary(10,5) -textvariable {} -variable lt(6) 
    vTcl:DefineAlias "$base.che46" "Checkbutton8" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che46 "$base.che46 Checkbutton $top all _vTclBalloon"
    bind $base.che46 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,5)
    }
    checkbutton $base.che47 \
        -selectcolor #ffffff -text $lary(10,6) -textvariable {} -variable lt(7) 
    vTcl:DefineAlias "$base.che47" "Checkbutton9" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che47 "$base.che47 Checkbutton $top all _vTclBalloon"
    bind $base.che47 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,6)
    }
    checkbutton $base.che48 \
        -selectcolor #ffffff -text $lary(10,7) -textvariable {} -variable lt(8) 
    vTcl:DefineAlias "$base.che48" "Checkbutton10" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che48 "$base.che48 Checkbutton $top all _vTclBalloon"
    bind $base.che48 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,7)
    }
    checkbutton $base.che49 \
        -selectcolor #ffffff -text $lary(10,0) -variable lf(1) 
    vTcl:DefineAlias "$base.che49" "Checkbutton11" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che49 "$base.che49 Checkbutton $top all _vTclBalloon"
    bind $base.che49 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,0)
    }
    checkbutton $base.che50 \
        -selectcolor #ffffff -text $lary(10,1) -variable lf(2) 
    vTcl:DefineAlias "$base.che50" "Checkbutton12" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che50 "$base.che50 Checkbutton $top all _vTclBalloon"
    bind $base.che50 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,1)
    }
    checkbutton $base.che51 \
        -command {if {$lf(3) == 1} {
    changekey 3
}} -selectcolor #ffffff \
        -text $lary(10,2) -variable lf(3) 
    vTcl:DefineAlias "$base.che51" "Checkbutton13" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che51 "$base.che51 Checkbutton $top all _vTclBalloon"
    bind $base.che51 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,2)
    }
    checkbutton $base.che52 \
        -command {if {$lf(4) == 1} {
    changekey 1
}} -selectcolor #ffffff \
        -text $lary(10,3) -variable lf(4) 
    vTcl:DefineAlias "$base.che52" "Checkbutton14" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che52 "$base.che52 Checkbutton $top all _vTclBalloon"
    bind $base.che52 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,3)
    }
    checkbutton $base.che53 \
        -command {if {$lf(5) == 1} {
    changekey 1
}} -selectcolor #ffffff \
        -text $lary(10,4) -variable lf(5) 
    vTcl:DefineAlias "$base.che53" "Checkbutton15" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che53 "$base.che53 Checkbutton $top all _vTclBalloon"
    bind $base.che53 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,4)
    }
    checkbutton $base.che54 \
        -command {if {$lf(6) == 1} {
    changekey 1
}} -selectcolor #ffffff \
        -text $lary(10,5) -variable lf(6) 
    vTcl:DefineAlias "$base.che54" "Checkbutton16" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che54 "$base.che54 Checkbutton $top all _vTclBalloon"
    bind $base.che54 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,5)
    }
    checkbutton $base.che56 \
        -command {if {$lf(7) == 1} {
    changekey 1
}} -selectcolor #ffffff \
        -text $lary(10,6) -variable lf(7) 
    vTcl:DefineAlias "$base.che56" "Checkbutton18" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che56 "$base.che56 Checkbutton $top all _vTclBalloon"
    bind $base.che56 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,6)
    }
    checkbutton $base.che57 \
        -command {if {$lf(8) == 1} {
    changekey 1
}} -selectcolor #ffffff \
        -text $lary(10,7) -variable lf(8) 
    vTcl:DefineAlias "$base.che57" "Checkbutton19" vTcl:WidgetProc "Toplevel1" 1
    bindtags $base.che57 "$base.che57 Checkbutton $top all _vTclBalloon"
    bind $base.che57 <<SetBalloon>> {
        set ::vTcl::balloon::%W $lary(11,7)
    }
    button $base.but61 \
        \
        -command {.top39.che49 select
.top39.che50 select
.top39.che51 select
.top39.che52 select
.top39.che53 select
.top39.che54 select
.top39.che56 select
.top39.che57 select} \
        -disabledforeground #a3a3a3 -pady 0 -text $lary(3,0)
    vTcl:DefineAlias "$base.but61" "Button2" vTcl:WidgetProc "Toplevel1" 1
    button $base.but62 \
        \
        -command {.top39.che49 deselect
.top39.che50 deselect
.top39.che51 deselect
.top39.che52 deselect
.top39.che53 deselect
.top39.che54 deselect
.top39.che56 deselect
.top39.che57 deselect} \
        -disabledforeground #a3a3a3 -pady 0 -text $lary(3,1)
    vTcl:DefineAlias "$base.but62" "Button3" vTcl:WidgetProc "Toplevel1" 1
    button $base.but63 \
        \
        -command {.top39.che40 select
.top39.che41 select
.top39.che43 select
.top39.che44 select
.top39.che45 select
.top39.che46 select
.top39.che47 select
.top39.che48 select} \
        -disabledforeground #a3a3a3 -height 16 -pady 0 -text $lary(3,0) -width 65 
    vTcl:DefineAlias "$base.but63" "Button4" vTcl:WidgetProc "Toplevel1" 1
    button $base.but64 \
        \
        -command {.top39.che40 deselect
.top39.che41 deselect
.top39.che43 deselect
.top39.che44 deselect
.top39.che45 deselect
.top39.che46 deselect
.top39.che47 deselect
.top39.che48 deselect} \
        -disabledforeground #a3a3a3 -height 16 -pady 0 -text $lary(3,1) -width 65 
    vTcl:DefineAlias "$base.but64" "Button5" vTcl:WidgetProc "Toplevel1" 1
    menu $base.m62 \
        -activeborderwidth 1 -borderwidth 1 -tearoff 1 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.tex40 \
        -x 5 -y 85 -width 650 -height 368 -anchor nw -bordermode ignore 
    place $base.lab42 \
        -x 5 -y 10 -width 54 -height 19 -anchor nw -bordermode ignore 
    place $base.ent45 \
        -x 70 -y 53 -width 231 -height 29 -anchor nw -bordermode ignore 
    place $base.lab46 \
        -x 3 -y 55 -width 54 -anchor nw -bordermode ignore 
    place $base.but70 \
        -x 310 -y 55 -anchor nw -bordermode ignore 
    place $base.lab71 \
        -x 455 -y 5 -width 50 -height 25 -anchor nw -bordermode ignore 
    place $base.lab72 \
        -x 455 -y 25 -width 50 -height 25 -anchor nw -bordermode ignore 
    place $base.che39 \
        -x 388 -y 55 -width 110 -height 27 -anchor nw -bordermode ignore 
    place $base.scr40 \
        -x 659 -y 85 -width 13 -height 369 -anchor nw -bordermode ignore 
    place $base.che40 \
        -x 65 -y 28 -width 50 -height 25 -anchor nw -bordermode ignore 
    place $base.che41 \
        -x 115 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che43 \
        -x 165 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che44 \
        -x 210 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che45 \
        -x 255 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che46 \
        -x 305 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che47 \
        -x 355 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che48 \
        -x 405 -y 28 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che49 \
        -x 65 -y 5  -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che50 \
        -x 115 -y 5 -width 50 -height 25 -anchor nw -bordermode ignore 
    place $base.che51 \
        -x 165 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che52 \
        -x 210 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che53 \
        -x 255 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che54 \
        -x 305 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che56 \
        -x 355 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.che57 \
        -x 405 -y 5 -width 50 -height 25  -anchor nw -bordermode ignore 
    place $base.but61 \
        -x 510 -y 10 -width 80 -height 18 -anchor nw -bordermode ignore 
    place $base.but62 \
        -x 600 -y 10 -width 80 -height 18 -anchor nw -bordermode ignore 
    place $base.but63 \
        -x 510 -y 30 -width 80 -height 18 -anchor nw -bordermode ignore 
    place $base.but64 \
        -x 600 -y 30 -width 80 -height 18 -anchor nw -bordermode ignore 

    vTcl:FireEvent $base <<Ready>>
}

#############################################################################
## Binding tag:  _TopLevel

bind "_TopLevel" <<Create>> {
    if {![info exists _topcount]} {set _topcount 0}; incr _topcount
}
bind "_TopLevel" <<DeleteWindow>> {
    destroy %W; if {$_topcount == 0} {exit}
}
bind "_TopLevel" <Destroy> {
    if {[winfo toplevel %W] == "%W"} {incr _topcount -1}
}
#############################################################################
## Binding tag:  _vTclBalloon


if {![info exists vTcl(sourcing)]} {
bind "_vTclBalloon" <<KillBalloon>> {
    namespace eval ::vTcl::balloon {
        after cancel $id
        if {[winfo exists .vTcl.balloon]} {
            destroy .vTcl.balloon
        }
        set set 0
    }
}
bind "_vTclBalloon" <<vTclBalloon>> {
    if {$::vTcl::balloon::first != 1} {break}

    namespace eval ::vTcl::balloon {
        set first 2
        if {![winfo exists .vTcl]} {
            toplevel .vTcl; wm withdraw .vTcl
        }
        if {![winfo exists .vTcl.balloon]} {
            toplevel .vTcl.balloon -bg black
        }
        wm overrideredirect .vTcl.balloon 1
        label .vTcl.balloon.l  -text ${%W} -relief flat  -bg #ffffaa -fg black -padx 2 -pady 0 -anchor w
        pack .vTcl.balloon.l -side left -padx 1 -pady 1
        wm geometry  .vTcl.balloon  +[expr {[winfo rootx %W]+[winfo width %W]/2}]+[expr {[winfo rooty %W]+[winfo height %W]+4}]
        set set 1
    }
}
bind "_vTclBalloon" <Button> {
    namespace eval ::vTcl::balloon {
        set first 0
    }
    vTcl:FireEvent %W <<KillBalloon>>
}
bind "_vTclBalloon" <Enter> {
    namespace eval ::vTcl::balloon {
        ## self defining balloon?
        if {![info exists %W]} {
            vTcl:FireEvent %W <<SetBalloon>>
        }
        set set 0
        set first 1
        set id [after 500 {vTcl:FireEvent %W <<vTclBalloon>>}]
    }
}
bind "_vTclBalloon" <Leave> {
    namespace eval ::vTcl::balloon {
        set first 0
    }
    vTcl:FireEvent %W <<KillBalloon>>
}
bind "_vTclBalloon" <Motion> {
    namespace eval ::vTcl::balloon {
        if {!$set} {
            after cancel $id
            set id [after 500 {vTcl:FireEvent %W <<vTclBalloon>>}]
        }
    }
}
}

Window show .
Window show .ktop
Window show .top39
main $argc $argv
